/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

// AddNoise.cpp: implementation of the AddNoise class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#include "AddNoise.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AddNoise::AddNoise()
{
	noiseWav = NULL;
}

AddNoise::~AddNoise()
{
	if (noiseWav != NULL) 
		delete noiseWav;
}

BOOL AddNoise::SetNoise(const char* noiseFilename)
{
	noisefile = fopen(noiseFilename,"rb");
	if (!noisefile)
	{
		CString stMsg;
		stMsg.Format("Cannot open noise file %s for read!\nIn <AddNoise::SetNoise()>.",noiseFilename);
		Exception(stMsg);
		return FALSE;
	}

	fseek(noisefile,0,SEEK_END);
	fl = ftell(noisefile);

	noiseLen = (fl-44)/2;
	fseek(noisefile,44,SEEK_SET);

	noiseWav = new short[noiseLen];
	if (noiseWav == NULL) 
	{
		Exception("Cannot alloc space for noise wave.\nIn <AddNoise::SetNoise()>.");
		return FALSE;
	}           
	
	fread(noiseWav,sizeof(short),noiseLen,noisefile);
	fclose(noisefile);

	return TRUE;
}

BOOL AddNoise::Add(short *inWav,int actLen,float snr)
{  
	int i;	
	int st;

	srand((unsigned)time(NULL));
	st = rand();
	st = st % noiseLen;
	st = 0;

	if(st+actLen > noiseLen)
		st -= actLen;
   
	float sum1=0.0;
	float sum2=0.0;
	float sum;
	float multi;
	float factor = 1.0/actLen;
	for(i=0; i<=actLen; i++)
	{
		sum1 += inWav[i]*inWav[i]*factor;
		sum2 += noiseWav[st+i]*noiseWav[st+i]*factor; 
	}
	if(sum2 > 0.1)
		sum = sum1/sum2;
	else
		sum = sum1*10;

	factor = 10*log10(sum);
	multi = (factor-snr)/10;
	factor = exp(0.5*multi*log(10));

	float maxm = 1.0/(factor+1.0);

	for (i=0; i<actLen; i++)
		inWav[i] = (short)((factor*noiseWav[st+i]+inWav[i])*maxm);

	return TRUE;
}

void AddNoise::Exception(const char* szMsg)
{
	AfxMessageBox(szMsg);
}