/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

// avsrDemoDoc.h : interface of the CAvsrDemoDoc class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_AVSRDEMODOC_H__665E0937_1190_4B51_8A94_4994DE6049C7__INCLUDED_)
#define AFX_AVSRDEMODOC_H__665E0937_1190_4B51_8A94_4994DE6049C7__INCLUDED_

#include "vfw.h"

#include "cvAvsrFeatureIO.h"
#include "CvAviIo.h"
#include "AddNoise.h"
#include "AudioMfcc.h"
#include "EvaluteResult.h"

#include "cvmouthtracking.h"
#include "cvvisualspeech.h"


#define	MAX_PATH_LEN	512
#define ORIG_WIN_SIZE	24

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CAvsrDemoDoc : public CDocument
{
protected: // create from serialization only
	CAvsrDemoDoc();
	DECLARE_DYNCREATE(CAvsrDemoDoc)

// Attributes
public:
	//CRunDemoThread m_thread;
// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAvsrDemoDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CAvsrDemoDoc();

	void SpeechRecognition();

	int  LoadAviFileList(const char *szPath);
	bool LoadMouthTracker(const char *szFileName); 
	bool LoadConfigFile(const char *szConfigFile);
	bool LoadTestData(const char *szAviFile);	
	
	bool InitSpeechModules();
	void FreeSpeechModules(void);

	bool NormalizeMouthRegion(const IplImage *iplSrcGrayImage, const bool bHistEqu, const bool bIgc, IplImage *iplRegionImage);
	bool VisualFrontEndInitial(const char *szPcaFile, const char *szLdaFile);
	bool VisualFrontEndFeatureExtract(const IplImage **piplImageArray, const int nFrameNum, const int nPcaOutDim, const float fUpsampleRate, const bool bDoFMN, const int nConWindowRadius, const float fFactor, CvMat* pmatOutput);
	bool VisualFrontEndUninitial();

	void GetResult(TWO_STR_ARRAY *&visual,TWO_STR_ARRAY *&audio,TWO_STR_ARRAY *&avsr);
	
	void SetStatusText(CString szText);

public:
	// video data
	BYTE *m_pDisplayVideo;

	CString m_szAviPath;
	CStringArray m_szAviList;
	char m_szMouthTrackCfg[1024];	// for mouth tracking config
	// demo initial files name
	char m_szPcaMatrixFile[1024];
	char m_szLdaMatrixFile[1024];
	char m_szMlltMatrixFile[1024];
	char m_szNoiseWaveFile[1024];
	char m_szVisualOnlyCfg[1024];
	char m_szAudioOnlyCfg[1024];
	char m_szAvsrCfg[1024];
	CvHidHaarClassifierCascade *m_phidCascadeFace;	// face detector
	CvHidHaarClassifierCascade *m_phidCascadeMouth0; // mouth detector0
	CvHidHaarClassifierCascade *m_phidCascadeMouth1; // mouth detector1
	CvMouthTracker *m_pHaarMouthTracker; // mouth tracker

	// to count the WER
	int m_nTotalWE_Vsr, m_nTotalWE_Asr, m_nTotalWE_Avsr;	// total error number
	int m_nTotalWordNum;				// total
	STR_ARRAY *m_pReference;			// current reference
	TWO_STR_ARRAY m_visualAlign, m_audioAlign, m_avsrAlign;
	
	// for visual feature IO
	CvAviIo	m_aviContainer;
	CAvsrFeatureDataIO m_featureIo;	
	bool m_isAviLoaded;
	// for PCA
	bool m_bPcaFileLoaded;
	CvMat *m_pmatPcaMeanVector;
	CvMat *m_pmatPcaEigenValue;
	CvMat *m_pmatPcaEigenVector;
	// for LDA
	bool m_bLdaFileLoaded;
	CvMat *m_pmatLdaMeanVector;
	CvMat *m_pmatLdaSdevVector;
	CvMat *m_pmatLdaTransformMatrix;
	CvMat *m_pmatLdaEigenvalue;

	// for audio feature
	AddNoise m_addNoise;
	float m_fSNR;			// signal to noise ratio
	short *m_pWave;	
	int	 m_nAudioMfccDimension;
	
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

// Generated message map functions
protected:
	//{{AFX_MSG(CAvsrDemoDoc)
	afx_msg void OnFileOpen();
	afx_msg void OnReset();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_AVSRDEMODOC_H__665E0937_1190_4B51_8A94_4994DE6049C7__INCLUDED_)
