/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

// avsrDemoView.cpp : implementation of the CAvsrDemoView class
//

#include "stdafx.h"
#include "avsrDemo.h"

#include "avsrDemoDoc.h"
#include "avsrDemoView.h"
#include "mainfrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define TO_BE_MODIFIED COLOR_WINDOWTEXT

#define RIGHT_COLOR RGB(0,0,255)
#define ORIGIN_COLOR RGB(0,0,0)
#define ERROR_COLOR RGB(128,0,0)
#define WORD_DELAY 50


/////////////////////////////////////////////////////////////////////////////
// CAvsrDemoView

IMPLEMENT_DYNCREATE(CAvsrDemoView, CFormView)

BEGIN_MESSAGE_MAP(CAvsrDemoView, CFormView)
	//{{AFX_MSG_MAP(CAvsrDemoView)
	ON_BN_CLICKED(IDC_START, OnStart)
	ON_BN_CLICKED(IDC_STOP, OnStop)
	ON_WM_HSCROLL()
	ON_BN_CLICKED(IDC_CHECK_NOISE, OnCheckNoise)
	ON_NOTIFY(NM_DBLCLK, IDC_LIST_AVI, OnDblclkListAvi)
	ON_MESSAGE(DRAW_END,OnDrawEnd)
	ON_WM_TIMER()
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CFormView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAvsrDemoView construction/destruction
CAvsrDemoView::CAvsrDemoView()
	: CFormView(CAvsrDemoView::IDD)
{
	//{{AFX_DATA_INIT(CAvsrDemoView)
	m_fSNR = 5;
	isShowResult =false;
	m_szAvsrError = _T("0.0");
	m_szVsrError = _T("0.0");
	m_szAsrError = _T("0.0");
	m_nNoise = 16;
	m_isBatch = FALSE;
	m_isNoise = TRUE;
	//}}AFX_DATA_INIT
	// TODO: add construction code here
	m_isSleep = true;
	m_isDone=false;
	m_isShowScript=false;
	m_isShowStack=false;
	m_isBatchStopped=false;
	m_isShowLastFrame=false;
	m_isPlaying=false;
	m_isNext=false;
	m_iplLastFrame=NULL;
	// initialize avi display window size
	m_nAviDisplayWidth = 360; 
	m_nAviDisplayHeight = 288;
}

CAvsrDemoView::~CAvsrDemoView()
{
	if (m_iplLastFrame != NULL){
		cvReleaseImage(&m_iplLastFrame);
		m_iplLastFrame = NULL;
	}
}

void CAvsrDemoView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAvsrDemoView)
	DDX_Control(pDX, IDC_LIST_AVI, m_cAviList);
	DDX_Control(pDX, IDC_CHECK_BATCH, m_cBatch);
	DDX_Control(pDX, IDC_CHECK_NOISE, m_cNoise);
	DDX_Control(pDX, IDC_STOP, m_cStop);
	DDX_Control(pDX, IDC_START, m_cStart);
	DDX_Control(pDX, IDC_RESET, m_cReset);
	DDX_Control(pDX, IDC_STATIC_PLAYER, m_cPlayer);
	DDX_Control(pDX, IDC_SLIDER_NOISE, m_cNoiseSlider);
	DDX_Control(pDX, IDC_STATIC_VSR_STACK, m_cVsrStack);
	DDX_Control(pDX, IDC_STATIC_AVSR_STACK, m_cAvsrStack);
	DDX_Control(pDX, IDC_STATIC_ASR_STACK, m_cAsrStack);
	DDX_Control(pDX, IDC_PROGRESS, m_pcrtProgress);
	DDX_Control(pDX, IDC_STATIC_SCRIPT, m_stScript);
	DDX_Control(pDX, IDC_STATIC_AVSR, m_stAVSR);
	DDX_Control(pDX, IDC_STATIC_ASR, m_stASR);
	DDX_Control(pDX, IDC_STATIC_VSR, m_stVSR);
	DDX_Text(pDX, IDC_NOISE_EDIT, m_fSNR);
	DDX_Control(pDX, IDC_AVIPLAYERCTRL, m_aviPlayer);
	DDX_Text(pDX, IDC_STATIC_AVSR_ERROR, m_szAvsrError);
	DDX_Text(pDX, IDC_STATIC_VSR_ERROR, m_szVsrError);
	DDX_Text(pDX, IDC_STATIC_ASR_ERROR, m_szAsrError);
	DDX_Check(pDX, IDC_CHECK_BATCH, m_isBatch);
	DDX_Check(pDX, IDC_CHECK_NOISE, m_isNoise);
	//}}AFX_DATA_MAP
}

BEGIN_EVENTSINK_MAP(CAvsrDemoView, CFormView)
    //{{AFX_EVENTSINK_MAP(CAvsrDemoView)
	//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()

BOOL CAvsrDemoView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CFormView::PreCreateWindow(cs);
}

void CAvsrDemoView::OnInitialUpdate()
{
	CFormView::OnInitialUpdate();
	GetParentFrame()->RecalcLayout();
	ResizeParentToFit();

	m_pcrtProgress.SetRange(0, 100);
	m_cNoiseSlider.SetRange(0,30);

	m_cNoiseSlider.SetPos(5);
}

/////////////////////////////////////////////////////////////////////////////
// CAvsrDemoView diagnostics

#ifdef _DEBUG
void CAvsrDemoView::AssertValid() const
{
	CFormView::AssertValid();
}

void CAvsrDemoView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CAvsrDemoDoc* CAvsrDemoView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CAvsrDemoDoc)));
	return (CAvsrDemoDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CAvsrDemoView message handlers
void CAvsrDemoView::OnStart() 
{
	// TODO: Add your control notification handler code here
	CAvsrDemoDoc *pDoc=(CAvsrDemoDoc*)GetDocument();
	if (!pDoc->m_isAviLoaded)
	{
		AfxMessageBox("load an avi first, please!\nIn <CAvsrDemoView::OnStart()>.");
		return;
	}
	Start();
}

void CAvsrDemoView::OnStop()
{
	// TODO: Add your control notification handler code here
	Stop();
}

void CAvsrDemoView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	// TODO: Add your message handler code here and/or call default
	UpdateData();
	m_fSNR=m_cNoiseSlider.GetPos();
	CAvsrDemoDoc *pDoc=GetDocument();
	pDoc->m_fSNR = m_fSNR;
	CFormView::OnHScroll(nSBCode, nPos, pScrollBar);
	UpdateData(false);
}

void CAvsrDemoView::OnCheckNoise() 
{
	// TODO: Add your control notification handler code here
	m_isNoise=!m_isNoise;
	CRect rect;
	if (!m_isNoise)
	{
		m_cNoiseSlider.EnableWindow(FALSE);
		GetDlgItem(IDC_NOISE_EDIT)->EnableWindow(FALSE);
		m_fSNR=1000.0;
		Invalidate(false);
	}else{
		m_cNoiseSlider.EnableWindow(TRUE);
		m_fSNR=m_cNoiseSlider.GetPos();
		GetDlgItem(IDC_NOISE_EDIT)->EnableWindow(TRUE);
		Invalidate(false);
	}
	CAvsrDemoDoc *pDoc=GetDocument();
	pDoc->m_fSNR = m_fSNR;
}

LRESULT CAvsrDemoView::OnDrawEnd(WPARAM, LPARAM)
{
	CMainFrame *pFrame=(CMainFrame*)AfxGetMainWnd();
	pFrame->m_wndStatusBar.SetPaneText(0,"Ready");
	m_isNext=false;
	CAvsrDemoDoc *pDoc=GetDocument();
	ProcessNext();
	return 1;
}

void CAvsrDemoView::OnDblclkListAvi(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
	int sel = m_cAviList.GetSelectionMark();
	m_cAviList.SetHotItem(sel);
	m_nOpenAviTimer=SetTimer(EVENT_OPEN_AVI,50,0);
	*pResult = 0;
}

void CAvsrDemoView::OnPaint() 
{
	CAvsrDemoDoc *pDoc = (CAvsrDemoDoc*)GetDocument();

	CFont SCRIPT_FONT;
	CPaintDC dc(this); // device context for painting
	CalcControllerPos();
	// TODO: Add your message handler code here
	if (m_isShowScript)
	{
		int nImageWidth, nImageHeight, nImageBand, nVideoLength;
		float fFrameRate;
		pDoc->m_aviContainer.GetVideoInfo(nImageWidth, nImageHeight, nImageBand, nVideoLength, fFrameRate);
		if (nVideoLength < 1) return;
		if (fFrameRate <= 0) return;
		STR_ARRAY *pReference = pDoc->m_pReference;
		if (pReference == NULL) return;
		int nTextNum = pReference->num;
		int nNonNullTextNum = 0;
		int nMaxTextLen = 4;
		char *pText;
		for (int i=0; i<nTextNum; i++) {
			pText = Arab2English(pReference->stra[i]);
			if (pText != NULL) {
				nNonNullTextNum++;
			}
		}
		int nRegionWidth = m_recTranscript.right-m_recTranscript.left;
		int nWordMargin = nRegionWidth / (2 * nNonNullTextNum + 1);	
		int nOffset = m_recTranscript.left + nWordMargin;
		int nFontWidth = nWordMargin / nMaxTextLen;		
		nFontWidth = (nFontWidth < 7) ? 7 : nFontWidth;
		nFontWidth = (nFontWidth > 14) ? 14 : nFontWidth;
		int nFontHeight= m_recTranscript.bottom - m_recTranscript.top - 16;
		dc.SetBkMode(TRANSPARENT);
		SCRIPT_FONT.CreateFont(nFontHeight, nFontWidth, 0, 0, FW_BOLD, false, false,
				0, ANSI_CHARSET,OUT_TT_PRECIS ,CLIP_CHARACTER_PRECIS,PROOF_QUALITY,FF_MODERN||DEFAULT_PITCH,NULL);
		CGdiObject *pOldFont = dc.SelectObject(&SCRIPT_FONT);
		COLORREF oldColor;
		char szText[16];
		int nPoint = 0, nTextnum;
		for (i=0; i<nTextNum; i++) {
			oldColor = dc.SetTextColor(ORIGIN_COLOR);
			pText = Arab2English(pReference->stra[i]);
			if (pText != NULL) {
				strncpy(szText, pText, 8);
				dc.DrawText(szText, CRect(nOffset+nPoint, m_recTranscript.top+12, m_recTranscript.right, m_recTranscript.bottom), 0);
				nTextnum = strlen(szText);
				if( nTextnum == 3)
					nPoint += 5 * nWordMargin / 3;
				else if ( nTextnum == 5)
					nPoint += 7 * nWordMargin / 3;
				else
					nPoint += 2 * nWordMargin;				
			}
			dc.SetTextColor(oldColor);
		}
	}
	if (isShowResult)
	{
		DrawResult(&dc);
		m_isSleep = false;
		m_isDone=true;
	}
	if (m_isShowStack)
	{
		DrawStack(&dc);
	}
	HDC hDC;
	HDRAWDIB hDib=DrawDibOpen();
	if (m_isShowLastFrame)
	{
		BITMAPINFO bmin;			
		bmin.bmiHeader.biSize = 40;
		bmin.bmiHeader.biWidth = m_iplLastFrame->width;
		bmin.bmiHeader.biHeight = m_iplLastFrame->height;
		bmin.bmiHeader.biSizeImage = m_iplLastFrame->imageSize;
		bmin.bmiHeader.biBitCount = m_iplLastFrame->nChannels * 8;
		bmin.bmiHeader.biPlanes = 1;
		bmin.bmiHeader.biCompression = 0;
		bmin.bmiHeader.biClrUsed = 0;
		bmin.bmiHeader.biClrImportant = 0;
		bmin.bmiHeader.biXPelsPerMeter = 100;
		bmin.bmiHeader.biYPelsPerMeter = 100;			
		
		hDC = dc.GetSafeHdc();
		DrawDibDraw(hDib,hDC,m_rectPlayer.left,m_rectPlayer.top,m_iplLastFrame->width,m_iplLastFrame->height,
			&(bmin.bmiHeader),m_iplLastFrame->imageData,0,0,m_iplLastFrame->width,m_iplLastFrame->height,0);
	}
	if (m_isNext)
		PostMessage(DRAW_END);
	// Do not call CFormView::OnPaint() for painting messages

}

void CAvsrDemoView::OnTimer(UINT nIDEvent) 
{
	// TODO: Add your message handler code here and/or call default
	CAvsrDemoDoc *pDoc=GetDocument();
	if (nIDEvent==EVENT_SPEECH_RECOGNITION)
	{
		KillTimer(m_nRunDemoTimer);
		pDoc->SpeechRecognition();
	}
	if (nIDEvent==EVENT_OPEN_AVI)
	{
		KillTimer(m_nOpenAviTimer);
		if(!pDoc->LoadTestData( pDoc->m_szAviList.GetAt(m_cAviList.GetSelectionMark()) ) ){			
			return;
		}
		if (m_isBatch)
			Start();
	}
	if (nIDEvent==EVENT_NEXT_AVI)
	{
		KillTimer(m_nNextAviTimer);
		if(!pDoc->LoadTestData( pDoc->m_szAviList.GetAt(m_cAviList.GetSelectionMark()) ) ){			
			return;
		}
		Start();
	}

	return;
}

void CAvsrDemoView::SetFileList(CStringArray &files, int nHot)
{
	m_cAviList.DeleteAllItems();
	int nAviNum = files.GetSize();
	for (int i=0; i<nAviNum; i++)
	{
		m_cAviList.InsertItem(i,files.GetAt(i));
	}
	m_cAviList.SetHotItem(nHot);
	m_cAviList.SetSelectionMark(nHot);

	return;
}

void CAvsrDemoView::CalcControllerPos()
{
	m_stAVSR.GetWindowRect(&m_recAVSRResult);
	m_stASR.GetWindowRect(&m_recASRResult);
	m_stVSR.GetWindowRect(&m_recVSRResult);
	m_stScript.GetWindowRect(&m_recTranscript);
	m_cAvsrStack.GetWindowRect(&m_recAvsrStack);
	m_cVsrStack.GetWindowRect(&m_recVsrStack);
	m_cAsrStack.GetWindowRect(&m_recAsrStack);
	m_cPlayer.GetWindowRect(&m_rectPlayer);

	CRect windowRect;
	GetWindowRect(&windowRect);
	m_recTranscript.left-=windowRect.left;
	m_recTranscript.top-=windowRect.top;
	m_recTranscript.bottom-=windowRect.top;
	m_recTranscript.right-=windowRect.left;
	m_recAVSRResult.left-=windowRect.left;
	m_recAVSRResult.top-=windowRect.top;
	m_recAVSRResult.bottom-=windowRect.top;
	m_recAVSRResult.right-=windowRect.left;
	m_recASRResult.left-=windowRect.left;
	m_recASRResult.top-=windowRect.top;
	m_recASRResult.bottom-=windowRect.top;
	m_recASRResult.right-=windowRect.left;
	m_recVSRResult.left-=windowRect.left;
	m_recVSRResult.top-=windowRect.top;
	m_recVSRResult.bottom-=windowRect.top;
	m_recVSRResult.right-=windowRect.left;
	m_recAsrStack.bottom-=windowRect.top;
	m_recAsrStack.right-=windowRect.left;
	m_recAsrStack.left-=windowRect.left;
	m_recAsrStack.top-=windowRect.top;
	m_recAvsrStack.bottom-=windowRect.top;
	m_recAvsrStack.right-=windowRect.left;
	m_recAvsrStack.left-=windowRect.left;
	m_recAvsrStack.top-=windowRect.top;
	m_recVsrStack.bottom-=windowRect.top;
	m_recVsrStack.right-=windowRect.left;
	m_recVsrStack.left-=windowRect.left;
	m_recVsrStack.top-=windowRect.top;
	m_rectPlayer.bottom-=windowRect.top-10;
	m_rectPlayer.right-=windowRect.left-2;
	m_rectPlayer.left-=windowRect.left-2;
	m_rectPlayer.top-=windowRect.top-10;
}

void CAvsrDemoView::Start()
{
	isShowResult=false;
	m_isPlaying=true;
	Invalidate();
	CMainFrame *pFrame=(CMainFrame*)AfxGetMainWnd();
	pFrame->m_wndStatusBar.SetPaneText(0,"Playing...");
	m_isBatchStopped=false;
	DisableController(true);
	((CListCtrl*)(GetDlgItem(IDC_LIST_AVI)))->EnableWindow(TRUE);
	CAvsrDemoDoc *pDoc=GetDocument();

	// get audio information
	int nAudioFrameNum, nAudioSampleSize, nAudioSampleRate;
	BYTE *pAudioData = pDoc->m_aviContainer.GetAudioData(nAudioFrameNum, nAudioSampleSize, nAudioSampleRate);
	if(pAudioData == NULL){
		AfxMessageBox("Get audio data failed!\nIn <CAvsrDemoView::Start()>.");
		return;
	}
	int nAudioLength = nAudioFrameNum * nAudioSampleSize / sizeof(short);
	memcpy(pDoc->m_pWave, pAudioData, nAudioLength*2);
	if (pDoc->m_pWave == NULL) {
		AfxMessageBox("Fail to load audio data!\nIn <CAvsrDemoView::Start()>.");
		return;
	}	
	// add acustic noise
	float fSNR = pDoc->m_fSNR / 2.0f;		// convert SNR(bubble ) to SNR(Gaussian)
	if (fSNR< 30) {
		pDoc->m_addNoise.Add(pDoc->m_pWave,nAudioLength, fSNR);
	}
	m_aviPlayer.SetAudioData((long)pDoc->m_pWave,nAudioSampleSize,nAudioLength);

	m_aviPlayer.Stop();
	// get video information
	int nImageWidth, nImageHeight, nImageBand, nVideoFrameNum;
	float fVideoFrameRate;
	if(!pDoc->m_aviContainer.GetVideoInfo(nImageWidth, nImageHeight, nImageBand, nVideoFrameNum, fVideoFrameRate)){
		AfxMessageBox("Get video info failed!\nIn <CAvsrDemoDoc::Start()>.");
		return;
	}	
	long time=long(nVideoFrameNum/fVideoFrameRate*1000);
	m_aviPlayer.Play();
	m_nRunDemoTimer = SetTimer(EVENT_SPEECH_RECOGNITION,time,0);
}

bool CAvsrDemoView::MakeResultImage(IplImage *iplShowImage, CvRect *pFace, CvRect *pMouth)
{
	if (iplShowImage == NULL || iplShowImage->width < 1 || iplShowImage->height < 1)
	{
		AfxMessageBox("Invalid input image!\nIn <CAvsrDemoDoc::MakeResultImage()>.");
		return false;
	}
	
	int nWidth = iplShowImage->width;
	int nHeight = iplShowImage->height;
	int nBand = iplShowImage->nChannels;
	int nLine = ((nWidth * nBand + 3) / 4) * 4;
	int x, y, left, right, top, bottom, i1, i2;

	// draw face rect
	if (pFace != NULL && (pFace->width) > 0 && (pFace->height) > 0) 
	{
		left = pFace->x < 0 ? 0 : pFace->x;
		left = left >= nWidth ? nWidth - 1 : left;
		int nright = pFace->width + pFace->x;
		right = nright < 0 ? 0 : nright;
		right = right >= nWidth ? nWidth - 1 : right;
		top = pFace->y < 0 ? 0 : pFace->y;
		top = top >= nHeight ? nHeight - 1 : top;
		int nbottom = pFace->y + pFace->height;
		bottom = nbottom < 0 ? 0 : nbottom;
		bottom = bottom >= nHeight ? nHeight - 1 : bottom;
		top = nHeight - top - 1;
		bottom = nHeight - bottom - 1;
		for (x=left; x<right; x++) 
		{
			i1 = top * nLine + x * 3;
			i2 = bottom * nLine + x * 3;
			iplShowImage->imageData[i1] = 0;
			iplShowImage->imageData[i1+1] = (char)255;
			iplShowImage->imageData[i1+2] = 0;
			iplShowImage->imageData[i2] = 0;
			iplShowImage->imageData[i2+1] = (char)255;
			iplShowImage->imageData[i2+2] = 0;
		}
		for (y=bottom; y<=top; y++)
		{
			i1 = y * nLine + left * 3;
			i2 = y * nLine + right * 3;
			iplShowImage->imageData[i1] = 0;
			iplShowImage->imageData[i1+1] = (char)255;
			iplShowImage->imageData[i1+2] = 0;
			iplShowImage->imageData[i2] = 0;
			iplShowImage->imageData[i2+1] = (char)255;
			iplShowImage->imageData[i2+2] = 0;
		}
	}
	// draw mouth rect
	if (pMouth != NULL && pMouth->width > 0 && pMouth->height > 0) 
	{
		left = pMouth->x < 0 ? 0 : pMouth->x;
		left = left >= nWidth ? nWidth - 1 : left;
		int nright = pMouth->width + pMouth->x;
		right = nright < 0 ? 0 : nright;
		right = right >= nWidth ? nWidth - 1 : right;
		top = pMouth->y < 0 ? 0 : pMouth->y;
		top = top >= nHeight ? nHeight - 1 : top;
		int nbottom = pMouth->y + pMouth->height;
		bottom = nbottom < 0 ? 0 : nbottom;
		bottom = bottom >= nHeight ? nHeight - 1 : bottom;
		top = nHeight - top - 1;
		bottom = nHeight - bottom - 1;
		for (x=left; x<right; x++) 
		{
			i1 = top * nLine + x * 3;
			i2 = bottom * nLine + x * 3;
			iplShowImage->imageData[i1] = 0;
			iplShowImage->imageData[i1+1] = 0;
			iplShowImage->imageData[i1+2] = (char)255;
			iplShowImage->imageData[i2] = 0;
			iplShowImage->imageData[i2+1] = 0;
			iplShowImage->imageData[i2+2] = (char)255;
		}
		for (y=bottom; y<=top; y++) 
		{
			i1 = y * nLine + left * 3;
			i2 = y * nLine + right * 3;
			iplShowImage->imageData[i1] = 0;
			iplShowImage->imageData[i1+1] = 0;
			iplShowImage->imageData[i1+2] = (char)255;
			iplShowImage->imageData[i2] = 0;
			iplShowImage->imageData[i2+1] = 0;
			iplShowImage->imageData[i2+2] = (char)255;
		}
	}

	return true;
}

void CAvsrDemoView::ShowSingleImage(IplImage *iplShowImage)
{
	CDC *pDC = GetDC();
	HDC hDC = pDC->GetSafeHdc();
	CalcControllerPos();
	int nImageWidth = iplShowImage->width;
	int nImageHeight = iplShowImage->height;
	int nImageBand = iplShowImage->nChannels;
	int nImageSize = iplShowImage->imageSize;
	BITMAPINFO bminfo;
	bminfo.bmiHeader.biSize = 40;		
	bminfo.bmiHeader.biWidth = nImageWidth;
	bminfo.bmiHeader.biHeight = nImageHeight;
	bminfo.bmiHeader.biBitCount = 8 * nImageBand;
	bminfo.bmiHeader.biSizeImage = nImageSize;
	bminfo.bmiHeader.biPlanes = 1;
	bminfo.bmiHeader.biCompression = 0;
	bminfo.bmiHeader.biClrUsed = 0;
	bminfo.bmiHeader.biClrImportant = 0;	
	bminfo.bmiHeader.biXPelsPerMeter = 100;
	bminfo.bmiHeader.biYPelsPerMeter = 100;
	int temp = SetDIBitsToDevice(hDC,m_rectPlayer.left,m_rectPlayer.top,
		nImageWidth,nImageHeight,0,0,0,nImageHeight,
		iplShowImage->imageData,&bminfo,DIB_RGB_COLORS);
}

void CAvsrDemoView::Stop()
{
	UpdateData();
	m_isBatchStopped=true;
	UpdateData(false);
}

void CAvsrDemoView::DrawResult(CDC *pDC)
{
	CAvsrDemoDoc *pDoc = GetDocument();
	if (pDoc == NULL) 
		return;

	CFont RIGHT_FONT,ERROR_FONT;
	int i;
	char szText[16];
	char *pText;
	int nPoint,nTextnum;

	// get evaluation results
	TWO_STR_ARRAY *pVisualAlign = NULL, *pAudioAlign = NULL, *pAvsrAlign = NULL;
	pDoc->GetResult(pVisualAlign, pAudioAlign, pAvsrAlign);
	if (pVisualAlign == NULL || pAudioAlign == NULL || pAvsrAlign == NULL) 
		return;

	// create font
	int nImageWidth, nImageHeight, nImageBand, nVideoLength;
	float fFrameRate;
	if(!pDoc->m_aviContainer.GetVideoInfo(nImageWidth, nImageHeight, nImageBand, nVideoLength, fFrameRate))
		return;
	if (nVideoLength < 1)
		return;
	if (fFrameRate <= 0)
		return;	
	int nNonNullTextNum1 = 0;
	int nNonNullTextNum2 = 0;
	int nNonNullTextNum3 = 0;
	int nNonNullTextNum;
	int nMaxTextLen = 4;
	for (i=0; i<pVisualAlign->num_a; i++) 
	{
		pText = Arab2English(pVisualAlign->str_b[i]);
		if (pText != NULL) 
			nNonNullTextNum1++;
	}
	for (i=0; i<pAudioAlign->num_a; i++) 
	{
		pText = Arab2English(pAudioAlign->str_b[i]);
		if (pText != NULL) 
			nNonNullTextNum2++;
	}
	for (i=0; i<pAvsrAlign->num_a; i++) 
	{
		pText = Arab2English(pAvsrAlign->str_b[i]);
		if (pText != NULL) 
			nNonNullTextNum3++;			
	}
	// get the max non-null text number
	if(nNonNullTextNum1 >= nNonNullTextNum2 && nNonNullTextNum1 >= nNonNullTextNum3)
		nNonNullTextNum = nNonNullTextNum1;
	else if(nNonNullTextNum2 >= nNonNullTextNum1 && nNonNullTextNum2 >= nNonNullTextNum3)
		nNonNullTextNum = nNonNullTextNum2;
	else
		nNonNullTextNum = nNonNullTextNum3;
	int nRegionWidth = m_recTranscript.right-m_recTranscript.left;
	int nWordMarginVisual = nRegionWidth / (2 * nNonNullTextNum1 + 1);	
	int nWordMarginAudio = nRegionWidth / (2 * nNonNullTextNum2 + 1);	
	int nWordMarginAvsr = nRegionWidth / (2 * nNonNullTextNum3 + 1);
	int nFontWidth = nRegionWidth / (2 * nNonNullTextNum + 1) / nMaxTextLen;
	int nOffset = m_recTranscript.left + nFontWidth * nMaxTextLen;
	nFontWidth = (nFontWidth < 7) ? 7 : nFontWidth;
	nFontWidth = (nFontWidth > 14) ? 14 : nFontWidth;
	int nFontHeight= m_recTranscript.bottom - m_recTranscript.top - 16;
	
	// set right text font
	RIGHT_FONT.CreateFont(nFontHeight,nFontWidth,0,0,FW_BOLD,false,false,
		0,ANSI_CHARSET,OUT_TT_PRECIS ,CLIP_CHARACTER_PRECIS,PROOF_QUALITY,FF_MODERN||DEFAULT_PITCH,NULL);
	// set error text font
	ERROR_FONT.CreateFont(nFontHeight,nFontWidth,0,0,FW_BOLD,false,false,
		0,ANSI_CHARSET,OUT_TT_PRECIS ,CLIP_CHARACTER_PRECIS,PROOF_QUALITY,FF_MODERN||DEFAULT_PITCH,NULL);
	
	pDC->SetBkMode(TRANSPARENT);
	CGdiObject *pOldFont = pDC->SelectObject(&RIGHT_FONT);
	COLORREF oldColor;

	// result of VSR
	nPoint = 0;
	for (i=0; i<pVisualAlign->num_a; i++)
	{
		if ((pVisualAlign->start_a[i]>=0) && (strcmp(pVisualAlign->str_a[i],pVisualAlign->str_b[i])==0))
			oldColor = pDC->SetTextColor(RIGHT_COLOR);
		else
			oldColor = pDC->SetTextColor(ERROR_COLOR);
		pText = Arab2English(pVisualAlign->str_b[i]);
		if (pText != NULL) 
		{
			strncpy(szText, pText, 8);
			pDC->DrawText(szText, CRect(nOffset+nPoint, m_recVSRResult.top+12, m_recVSRResult.right, m_recVSRResult.bottom), 0);
			nTextnum = strlen(szText);
			if( nTextnum == 3)
				nPoint += 5 * nWordMarginVisual / 3;
			else if ( nTextnum == 5)
				nPoint += 7 * nWordMarginVisual / 3;
			else
				nPoint += 2 * nWordMarginVisual;
		}
		if (m_isSleep) 
			Sleep(WORD_DELAY);
		pDC->SetTextColor(oldColor);
	}
	if (m_isSleep) 
		Sleep(WORD_DELAY*10);

	// result of ASR
	nPoint = 0;
	for (i=0; i<pAudioAlign->num_a; i++)
	{
		if ((pAudioAlign->start_a[i]>=0) && (strcmp(pAudioAlign->str_a[i],pAudioAlign->str_b[i])==0))
			oldColor = pDC->SetTextColor(RIGHT_COLOR);
		else
			oldColor = pDC->SetTextColor(ERROR_COLOR);
		pText = Arab2English(pAudioAlign->str_b[i]);
		if (pText != NULL) 
		{
			strncpy(szText, pText, 8);
			pDC->DrawText(szText, CRect(nOffset+nPoint, m_recASRResult.top+12, m_recASRResult.right, m_recASRResult.bottom), 0);
			nTextnum = strlen(szText);
			if( nTextnum == 3)
				nPoint += 5 * nWordMarginAudio / 3;
			else if ( nTextnum == 5)
				nPoint += 7 * nWordMarginAudio / 3;
			else
				nPoint += 2 * nWordMarginAudio;			
		}
		if (m_isSleep) Sleep(WORD_DELAY);
		pDC->SetTextColor(oldColor);
	}
	if (m_isSleep) Sleep(WORD_DELAY*10);

	// result of AVSR
	nPoint = 0;
	for (i=0; i<pAvsrAlign->num_a; i++)
	{
		if ((pAvsrAlign->start_a[i]>=0) && (strcmp(pAvsrAlign->str_a[i],pAvsrAlign->str_b[i])==0))
			oldColor = pDC->SetTextColor(RIGHT_COLOR);
		else
			oldColor = pDC->SetTextColor(ERROR_COLOR);
		pText = Arab2English(pAvsrAlign->str_b[i]);
		if (pText != NULL) 
		{
			strncpy(szText, pText, 8);
			pDC->DrawText(szText, CRect(nOffset+nPoint, m_recAVSRResult.top+12, m_recAVSRResult.right, m_recAVSRResult.bottom), 0);
			nTextnum = strlen(szText);
			if( nTextnum == 3)
				nPoint += 5 * nWordMarginAvsr / 3;
			else if ( nTextnum == 5)
				nPoint += 7 * nWordMarginAvsr / 3;
			else
				nPoint += 2 * nWordMarginAvsr;
		}
		if (m_isSleep) 
			Sleep(WORD_DELAY);
		pDC->SetTextColor(oldColor);
	}
}

void CAvsrDemoView::DrawStack(CDC *pDC)
{
	CAvsrDemoDoc *pDoc=GetDocument();
	int nTotalWordNum = pDoc->m_nTotalWordNum;
	float fWER_Avsr, fWER_Asr, fWER_Vsr;
	if (nTotalWordNum == 0)
	{
		fWER_Avsr = fWER_Asr = fWER_Vsr = 0.0f;
	}else{
		fWER_Avsr = ((float)(pDoc->m_nTotalWE_Avsr)) / nTotalWordNum;
		fWER_Vsr = ((float)(pDoc->m_nTotalWE_Vsr)) / nTotalWordNum;
		fWER_Asr = ((float)(pDoc->m_nTotalWE_Asr)) / nTotalWordNum;
	}

	UpdateData();
	m_szAvsrError.Format(_T("%.1f%%"), fWER_Avsr*100);
	m_szAsrError.Format(_T("%.1f%%"), fWER_Asr*100);
	m_szVsrError.Format(_T("%.1f%%"), fWER_Vsr*100);
	UpdateData(false);
	COLORREF avsrColor,vsrColor,asrColor, backColor;

	CRect avsrError,asrError,vsrError;
	backColor = RGB(212, 208, 200);
	CBrush backBrush(backColor);
	avsrError.bottom=m_recAvsrStack.bottom-4;
	avsrError.top=m_recAvsrStack.top+10;
	avsrError.left=m_recAvsrStack.left+1;
	avsrError.right=m_recAvsrStack.right-5;
	vsrError.bottom=m_recVsrStack.bottom-4;
	vsrError.top=m_recVsrStack.top+10;
	vsrError.left=m_recVsrStack.left+1;
	vsrError.right=m_recVsrStack.right-5;
	asrError.bottom=m_recAsrStack.bottom-4;
	asrError.top=m_recAsrStack.top+10;
	asrError.left=m_recAsrStack.left+1;
	asrError.right=m_recAsrStack.right-5;
	pDC->FillRect(avsrError,&backBrush);
	pDC->FillRect(asrError,&backBrush);
	pDC->FillRect(vsrError,&backBrush);

	// draw new stack
	avsrColor=RGB(255, 255, 0);	
	asrColor=RGB(224,0, 224);
	vsrColor=RGB(0, 0, 255);
	CBrush avsrBrush(avsrColor),asrBrush(asrColor),vsrBrush(vsrColor);
	float fAvsrH = fWER_Avsr > 1 ? 0 : 1-fWER_Avsr;
	avsrError.top=avsrError.top+fAvsrH*(avsrError.bottom-avsrError.top);
	float fVsrH = fWER_Vsr > 1 ? 0 : 1-fWER_Vsr;
	vsrError.top=vsrError.top+fVsrH*(vsrError.bottom-vsrError.top);
	float fAsrH = fWER_Asr > 1 ? 0 : 1-fWER_Asr;
	asrError.top=asrError.top+fAsrH*(asrError.bottom-asrError.top);
	pDC->FillRect(avsrError,&avsrBrush);
	pDC->FillRect(vsrError,&vsrBrush);
	pDC->FillRect(asrError,&asrBrush);

}

void CAvsrDemoView::ProcessNext()
{
	int nNext;
	m_pcrtProgress.SetPos(0);
	POINT prePos,curPos;
	if (m_isBatch)
	{
		nNext=m_cAviList.GetSelectionMark();
		
		if ((nNext<((CListCtrl*)(GetDlgItem(IDC_LIST_AVI)))->GetItemCount()-1)&&(!m_isBatchStopped))
		{
			m_cAviList.GetItemPosition(nNext,&prePos);
			nNext++;
			m_cAviList.GetItemPosition(nNext,&curPos);
			if (curPos.x>prePos.x)
				m_cAviList.Scroll(CSize(curPos.x-prePos.x,0));
			m_cAviList.SetHotItem(nNext);
			m_cAviList.SetSelectionMark(nNext);
			m_nNextAviTimer = SetTimer(EVENT_NEXT_AVI,50,0);
		}else{
			m_isPlaying=false;
			Stop();
			CMainFrame *pFrame=(CMainFrame*)AfxGetMainWnd();
			CAvsrDemoDoc *pDoc=GetDocument();
			pDoc->SetStatusText("Ready");
			DisableController(false);
		}

	}else{
		m_isPlaying=false;
		DisableController(false);
	}
	
}

void CAvsrDemoView::DisableController(bool toBeDisabled)
{
	if (toBeDisabled)
	{
		m_cNoiseSlider.EnableWindow(FALSE);
		m_cStart.EnableWindow(FALSE);
		m_cNoise.EnableWindow(FALSE);
		m_cBatch.EnableWindow(FALSE);
		m_cAviList.EnableWindow(FALSE);
		m_cReset.EnableWindow(FALSE);
	}else{
		m_cNoiseSlider.EnableWindow(TRUE);
		m_cStart.EnableWindow(TRUE);
		m_cNoise.EnableWindow(TRUE);
		m_cBatch.EnableWindow(TRUE);
		m_cAviList.EnableWindow(TRUE);
		m_cReset.EnableWindow(TRUE);
	}
}

void CAvsrDemoView::SetLastFrame(IplImage *iplImage)
{
	if(iplImage == NULL){
		AfxMessageBox("Invalid input frame!\nIn <CAvsrDemoView::SetLastFrame()>.");
		return;
	}
	if (m_iplLastFrame != NULL){
		cvReleaseImage(&m_iplLastFrame);
		m_iplLastFrame = NULL;
	}	
	m_iplLastFrame = cvCloneImage(iplImage);
}

char *CAvsrDemoView::Arab2English(char *szArab)
{
	static char english_digit[10][16]={"zero","one","two","three","four","five","six","seven","eight","nine"};
	static char arab_digit[10][2]={"0","1","2","3","4","5","6","7","8","9"};

	for (int i=0; i<10; i++) {
		if (strcmp(szArab, arab_digit[i]) == 0) {
			return english_digit[i];
		}
	}

	return NULL;
}

