/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#include "stdafx.h"
#include <stdlib.h> 
#include <malloc.h>
#include "EvaluteResult.h"

#define MAX_SYMBS_IN_STR		128
#define MAX_LABEL_STR_LEN		128

int overlapframe = 0;

#define Max(a,b) ((a>b)?a:b)
#define Min(a,b) ((a>b)?b:a)

int	Max4Int(int a, int b, int c, int d);

static enum edits PickMin(int dx[])
{
	enum edits choice;

	// pick min of dx[edits] using special logic to break ties
	if (dx[DEL] < dx[SUB]) {
		if (dx[DEL] < dx[INS]) 
			choice = DEL;
		else {
			if (dx[INS] < dx[DEL]) 
				choice = INS;
			else {
				// dx[DEL],dx[INS] tie
                choice = INS; // arb it
			}
		}
	}
	else // dx[DEL] >= dx[SUB]
    {
		if (dx[DEL] == dx[SUB]) {
			if (dx[INS] < dx[DEL]) 
				choice = INS;
			else {
				if (dx[INS] > dx[DEL]) {
					// dx[DEL],dx[SUB] tie
					// pick SUB
					choice = SUB;
				}
				else {
					// dx[DEL], dx[SUB], dx[INS] tie
					// pick SUB
					choice = SUB;
				}
			}  
		}
		else
		{// dx[DEL] is out of contention
			if (dx[SUB] < dx[INS]) 
				choice = SUB;
			else {
				if (dx[INS] < dx[SUB]) 
					choice = INS;
				else {
					// dx[SUB], dx[INS] tie
					// pick SUB
					choice = SUB;
				}  
			}  
		}  
	}
	return choice;
}

int CalcDist(char *ref_str, char *hyp_str,int start_r,int end_r,int start_h,int end_h)
{
	int distance=0;
	if(ref_str==NULL && hyp_str==NULL) {
		distance=0;
		return(distance);
	}
	else if (ref_str!=NULL && hyp_str!=NULL && strcmp(ref_str,hyp_str)==0 && start_r !=-1 && 
			 end_r !=-1 && start_h !=-1 && end_h !=-1 && (Min(end_r,end_h)-Max(start_r,start_h) >=overlapframe)) {
		distance=0;
		return(distance);
	}

	if( ref_str==NULL || hyp_str==NULL) {
		distance = 3;
	}
	else 
		distance = 4;

	return distance;
}

int DoWordAlignment(STR_ARRAY hyp_array, STR_ARRAY ref_array, TWO_STR_ARRAY *align_result,
					  int &accuracy, int &insert_num, int &delete_num, int &sub_num)
{
	accuracy=0;
	insert_num=0;
	delete_num=0;
	sub_num=0;

	int i,j,k;
	
	align_result->num = hyp_array.num + ref_array.num;

	//allocate memory
	align_result->str_a = new char* [align_result->num];
	align_result->start_a = new int [align_result->num];
	align_result->end_a = new int [align_result->num];
	align_result->str_b = new char* [align_result->num];
	align_result->start_b = new int [align_result->num];
	align_result->end_b = new int [align_result->num];
	if (align_result->end_b == NULL || align_result->start_a == NULL || align_result->end_a == NULL ||
		align_result->str_a == NULL || align_result->str_b == NULL || align_result->start_b == NULL) {
		AfxMessageBox("Fail to allocate string array!\n");
		return -1;
	}

	for(i=0; i<align_result->num; i++) {
		align_result->str_a[i] = new char [64];
		align_result->str_b[i] = new char [64];
		if (align_result->str_a[i] == NULL || align_result->str_b[i] == NULL) {
			AfxMessageBox("Fail to allocate string array!\n");
			return -1;
		}
	}
	//align
   int d[MAX_SYMBS_IN_STR][MAX_SYMBS_IN_STR], distance, dx[4];
   int back_pointer[MAX_SYMBS_IN_STR][MAX_SYMBS_IN_STR];
   enum edits choice;

   distance=0;
   //check parameters
   if(hyp_array.num == 0 && ref_array.num == 0) {
	   AfxMessageBox("Error: Null strings\n");
	   align_result->num_a = 0;
	   align_result->num_b = 0;
	   return distance;
   }
   //ref array length==0
   if (ref_array.num == 0) {
      for (j=0; j<hyp_array.num; j++) {
		  distance += CalcDist(NULL,hyp_array.stra[j],-1,-1,hyp_array.start[j],hyp_array.end[j]);
          align_result->str_a[j][0] = '\0';
		  align_result->start_a[j]=-1;
		  align_result->end_a[j]=-1;
          strcpy(align_result->str_b[j], hyp_array.stra[j]);
		  align_result->start_b[j]=hyp_array.start[j];
		  align_result->end_b[j]=hyp_array.end[j];
      }
      align_result->num_a = hyp_array.num;
	  align_result->num_b = hyp_array.num;   
      return distance;
   }
   //hyp array length==0
   if (hyp_array.num == 0) {
      for (j=0; j<ref_array.num; j++) {
		  distance += CalcDist(ref_array.stra[j],NULL,ref_array.start[j],ref_array.end[j],-1,-1);
          align_result->str_b[j][0] = '\0';
		  align_result->start_b[j]=-1;
		  align_result->end_b[j]=-1;
          strcpy(align_result->str_a[j], ref_array.stra[j]);
		  align_result->start_a[j]=ref_array.start[j];
		  align_result->end_a[j]=ref_array.end[j];
      }
      align_result->num_a = ref_array.num;
	  align_result->num_b = ref_array.num;   
      return distance;
   }
   //normal case
   d[0][0] = 0;

   for (i = 1; i <= ref_array.num; i++) {
		d[i][0] = d[i-1][0] + CalcDist(ref_array.stra[i-1],NULL,ref_array.start[i-1],ref_array.end[i-1],-1,-1);
		back_pointer[i][0] = DEL;
   }
   for (j = 1; j <= hyp_array.num; j++) {
		d[0][j] = d[0][j-1] + CalcDist(NULL, hyp_array.stra[j-1],-1,-1,hyp_array.start[j-1],hyp_array.end[j-1]);
		back_pointer[0][j] = INS;
   }
   //body cells
   for (j = 1; j <= hyp_array.num; j++) {
	   for (i = 1; i <= ref_array.num; i++) {
			dx[DEL] = d[i-1][j]   + d[i][0] - d[i-1][0];
			dx[SUB] = d[i-1][j-1] + CalcDist(ref_array.stra[i-1],hyp_array.stra[j-1],ref_array.start[i-1],ref_array.end[i-1],hyp_array.start[j-1],hyp_array.end[j-1]); /*sub*/
			dx[INS] = d[i][j-1]   + d[0][j] - d[0][j-1];
			choice = PickMin(dx);
			d[i][j] = dx[choice];
			back_pointer[i][j] = choice;
		}
   }
   distance = d[ref_array.num][hyp_array.num];

   k = 0;
   i = ref_array.num;
   j = hyp_array.num;
   while ((i > 0) || (j > 0))
   {
	   k += 1;
       if (back_pointer[i][j] == DEL) {
		   strcpy(align_result->str_a[k-1], ref_array.stra[i-1]);
		   align_result->start_a[k-1]=ref_array.start[i-1];
		   align_result->end_a[k-1]=ref_array.end[i-1];

           align_result->str_b[k-1][0]='\0';
		   align_result->start_b[k-1]=-1;
		   align_result->end_b[k-1]=-1;
           --i;
       }
       else
       {
			if (back_pointer[i][j] == SUB) {
				strcpy(align_result->str_a[k-1], ref_array.stra[i-1]);
		        align_result->start_a[k-1]=ref_array.start[i-1];
		        align_result->end_a[k-1]=ref_array.end[i-1];

				strcpy(align_result->str_b[k-1], hyp_array.stra[j-1]);
		        align_result->start_b[k-1]=hyp_array.start[j-1];
		        align_result->end_b[k-1]=hyp_array.end[j-1];

				--i;
				--j;
            }
            else
            {
				if (back_pointer[i][j] == INS) {
					align_result->str_a[k-1][0]='\0';
		            align_result->start_a[k-1]=-1;
		            align_result->end_a[k-1]=-1;

					strcpy(align_result->str_b[k-1], hyp_array.stra[j-1]);
		            align_result->start_b[k-1]=hyp_array.start[j-1];
		            align_result->end_b[k-1]=hyp_array.end[j-1];

					--j;
                }
				else
				{
					AfxMessageBox("Error: back pointer=%d\n",back_pointer[i][j]);
					return distance;
				}
			}
		}
	}
    align_result->num_a = k;
    align_result->num_b = k;

    j = k-1;
	char tempa[64],tempb[64];
	int start_a,end_a,start_b,end_b;
    for (i = 0; i <= (k / 2)-1; i++) {
		strcpy(tempa, align_result->str_a[i]);
		start_a=align_result->start_a[i];
		end_a=align_result->end_a[i];

		strcpy(tempb, align_result->str_b[i]);
		start_b=align_result->start_b[i];
		end_b=align_result->end_b[i];

        strcpy(align_result->str_a[i], align_result->str_a[j]);
		align_result->start_a[i] = align_result->start_a[j];
		align_result->end_a[i] = align_result->end_a[j];

		strcpy(align_result->str_b[i], align_result->str_b[j]);
		align_result->start_b[i] = align_result->start_b[j];
		align_result->end_b[i] = align_result->end_b[j];

		strcpy(align_result->str_a[j],tempa);
		align_result->start_a[j] = start_a;
		align_result->end_a[j] = end_a;

		strcpy(align_result->str_b[j],tempb);
		align_result->start_b[j] = start_b;
		align_result->end_b[j] = end_b;

        j--;
    }
	// release part of buffer
	for (i=k; i<align_result->num; i++) {
		if (align_result->str_a[i] != NULL) delete align_result->str_a[i];
		if (align_result->str_b[i] != NULL) delete align_result->str_b[i];
	}
	align_result->num = k;

	for(i=0; i<k; i++) {
		if(align_result->str_a[i][0]!='\0' && strcmp(align_result->str_a[i], align_result->str_b[i])==0 \
			&& align_result->end_a[i] != -1 && align_result->end_b[i] != -1 && align_result->start_a[i] != -1 && align_result->start_b[i] != -1 \
			&& (Min(align_result->end_a[i],align_result->end_b[i])-Max(align_result->start_a[i],align_result->start_b[i]) >=overlapframe) )
			accuracy++;
		else if(align_result->str_a[i][0]=='\0' && align_result->str_b[i][0]!='\0' )
				insert_num++;
	    else if(align_result->str_a[i][0]!='\0' && align_result->str_b[i][0]=='\0' )
			delete_num++;
		else 
			sub_num++;
	}

	return distance;
}

void GetScriptFilename(const char *szSrcFile, char *szDataFile, const char *szEx)
{
	int avi='.';
	memset(szDataFile,0,sizeof(szDataFile));
	char* ex = strrchr(szSrcFile,avi);
	int length = ex-szSrcFile+1;
	int i,j;
	for (i=0;i<length;i++)
	{
		szDataFile[i]=szSrcFile[i];
	}
	for (j=0;j<sizeof(szEx);j++)
	{
		szDataFile[i+j]=szEx[j];
	}
	szDataFile[i+j]=0;
}


STR_ARRAY *LoadScriptFile(char szFilename[])
{
	static char english_digit[10][16]={"zero","one","two","three","four","five","six","seven","eight","nine"};
	static char arab_digit[10][2]={"0","1","2","3","4","5","6","7","8","9"};

	FILE *fp = fopen(szFilename, "rt");
	if (fp == NULL) {
		AfxMessageBox("Fail to open transcript file!\n");
		return NULL;
	}

	// allocate memory
	STR_ARRAY *pRef = new STR_ARRAY;
	pRef->num = 0;	
	pRef->stra = new char* [MAX_LABEL_STR_LEN];
	pRef->start = new int [MAX_LABEL_STR_LEN];
	pRef->end = new int [MAX_LABEL_STR_LEN];

	// load label
	int i;
	char szLine[256];
	fgets(szLine, 256, fp);		// blank line
	fgets(szLine, 256, fp);		// blank line
	do {
		if (fgets(szLine, 256, fp) == NULL) break;		// data line
		int nStart, nEnd;
		char szDigit[256];
		if (sscanf(szLine, "%d %d %s ", &nStart, &nEnd, szDigit) != 3) {
			break;
		}
		// convert
		bool isData = false;
		for (i=0; i<10; i++) {
			if (strcmp(szDigit, english_digit[i]) == 0) {
				pRef->stra[pRef->num] = new char[16];
				strcpy(pRef->stra[pRef->num], arab_digit[i]);
				pRef->start[pRef->num] = nStart;
				pRef->end[pRef->num] = nEnd;
				pRef->num ++;
				isData = true;
				break;
			}
		}
		if (!isData) break;
	} while (!feof(fp));

	fclose(fp);

	if (pRef->num < 1) {
		ReleaseStrArray(pRef);
		delete pRef;
		return NULL;
	}

	return pRef;
}

STR_ARRAY *FormatResult(char szResult[])
{
	// allocate memory
	int i;
	STR_ARRAY *pRef = new STR_ARRAY;
	pRef->num = 0;
	pRef->stra = new char* [MAX_LABEL_STR_LEN];
	for (i=0; i<MAX_LABEL_STR_LEN; i++) {
		pRef->stra[i] = NULL;
	}
	pRef->start = new int [MAX_LABEL_STR_LEN];
	pRef->end = new int [MAX_LABEL_STR_LEN];

	int nLen = strlen(szResult);
	int loc = 0;
	int next = 0;
	char szBuf[64];
	while (loc < nLen) {
		// get segment
		i = 0;
		while (next < nLen && szResult[next] != ' ') {
			szBuf[i] = szResult[next];
			i++;
			next ++;
		}
		szBuf[i] = 0;
		next ++;
		loc = next;
		// read data
		int nStart, nEnd;
		int nDigit;
		if (sscanf(szBuf, "%d[%d,%d]", &nDigit, &nStart, &nEnd) == 3) {
			if (pRef->num >= MAX_LABEL_STR_LEN) {			
				break;
			}
			pRef->stra[pRef->num] = new char[16];
			sprintf(pRef->stra[pRef->num], "%1d", nDigit);
			pRef->start[pRef->num] = nStart;
			pRef->end[pRef->num] = nEnd;
			pRef->num ++;
		}
	}

	return pRef;
}

void ReleaseStrArray(STR_ARRAY *pRef)
{
	if (pRef == NULL) 
		return;
	if (pRef->num < 1) 
		return;

	for (int i=0; i<pRef->num; i++) 
	{
		if (pRef->stra[i] != NULL) 
			delete pRef->stra[i];
	}
	delete pRef->stra;
	if (pRef->start != NULL)
		delete pRef->start;
	if (pRef->end != NULL)
		delete pRef->end;
}

void Release2StrArray(TWO_STR_ARRAY *result)
{
	if (result == NULL)
		return;
	if (result->num < 1) 
		return;

	if (result->start_a != NULL) 
		delete result->start_a;
	if (result->start_b != NULL) 
		delete result->start_b;
	if (result->end_a != NULL) 
		delete result->end_a;
	if (result->end_b != NULL) 
		delete result->end_b;
	for(int i=0; i<result->num; i++)
	{
		if (result->str_a[i] != NULL) 
			delete result->str_a[i];
		if (result->str_b[i] != NULL) 
			delete result->str_b[i];
	}
	delete result->str_a;
	delete result->str_b;
}

void Empty2StrArray(TWO_STR_ARRAY &result)
{
	result.num = 0;
	result.start_a = NULL;
	result.start_b = NULL;
	result.end_a = NULL;
	result.end_b = NULL;
	result.str_a = NULL;
	result.str_b = NULL;
}

int	Max4Int(int a, int b, int c, int d)
{
	int max1 = a > b ? a : b;
	int max2 = c > d ? c : d;
	if (max1 > max2) 
		return max1;
	else 
		return max2;
}

