/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#ifndef	_INTEL_ICRC_AVSR_AUDIO_MFCC_WRAPPER_H
#define _INTEL_ICRC_AVSR_AUDIO_MFCC_WRAPPER_H

#ifdef __cplusplus
extern "C" {
#endif

__declspec(dllexport) bool InitMFCC();

/*F///////////////////////////////////////////////////////////////////////////////////////
//
//    Name: audio_mfcc_transform(short *pWaveData,int nSmpCount, float *pMfcc, int &nFeatureNum, long lSmpRate)
//    Purpose:
//			Do MFCC transform. Get 13-D MFCC feature, 10 ms per sample.
//    Context:
//    Parameters:
//			pWaveData	---- [I] input wave data, linear array
//			nSmpCount	---- [I] sample number of the input wave data
//			pMfcc		---- [O] extracted MFCC feature data, 2-D array, 13-D MFCC features frame by frame;
//								 must be allocated before this function called. The size should be larger than 
//								 float[13*100*nSmpCount/lSmpRate]
//			nFeatureNum ---- [I/O] frame number of the MFCC feature data, input the limitation, output the real number
//			lSmpRate	---- [I] sample rate of the input wave data, could only be 16000Hz, 48000Hz and 44100Hz
//    Returns: 
//		  return the data buffer
//F*/
__declspec(dllexport) bool ExtractMFCC(short *pWaveData,int nSmpCount, float *pMfcc, int &nFeatureNum, long lSmpRate);

__declspec(dllexport) bool ReleaseMFCC(void);


#ifdef __cplusplus
}
#endif


#endif	//#define _INTEL_ICRC_AVSR_AUDIO_MFCC_WRAPPER_H