/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/


#ifndef _INTEL_ICRC_AVSR_AVI_LOADER_H
#define _INTEL_ICRC_AVSR_AVI_LOADER_H

#include <afxwin.h> 
#include "cv.h"

#define		AVSR_MAX_AVI_LENGTH_IN_MEMORY		512

enum { 
	Audio_LChannel,
	Audio_RChannel,
	Audio_BChannel
};

class CvAviIo {
public:
	CvAviIo(void);
	~CvAviIo(void);

protected:
	// avi loading control
	bool		m_isAudioLoad;
	bool		m_isVideoLoad;
	char		m_szCurrentAVIFile[256];
	int			m_nCurrentFrameNumber;			// current frame number in the memory
	int			m_nStartFrameID;				// the global frame no. of the first frame in the memory
	// video data
	int			m_nImageWidth;
	int			m_nImageHeight;
	int			m_nImageBand;
	float		m_fVideoFrameRate;
	int			m_nVideoFrameNum;				// total frame number of the AVI file
	IplImage	**m_pVideoData;
	// audio data
	int			m_nAudioFrameNum;
	int			m_nAudioSampleRate;
	int			m_nAudioSampleSize;
	BYTE	   *m_pAudioData;

	int			LoadVideoFrame(const char szFileName[], int nStart, int nEnd);
	bool		ReadWavFromAvi(const char lpszAVIFileName[], int nMode=Audio_LChannel);

public:
	bool		LoadAvi(const char szFileName[], bool bAudio = true, bool bVideo = true);

	bool		GetAviFilename(char szAviFile[]) {
		if (!m_isAudioLoad && !m_isVideoLoad) return FALSE;
		strncpy(szAviFile, m_szCurrentAVIFile, 255);
		return TRUE;
	}

	IplImage *	GetImage(int nFrameNum);
	BYTE	 *	GetAudioData(int &nFrameNum, int &SampleSize, int &nSampleRate) {
		if (!m_isAudioLoad) return NULL;
		nFrameNum = m_nAudioFrameNum;
		nSampleRate = m_nAudioSampleRate;
		SampleSize = m_nAudioSampleSize;
		return m_pAudioData;
	}

	bool		GetVideoInfo(int &nImageWidth, int &nImageHeight, int &nImageBand, int &nFrameNum, float &fFrameRate);

	void		ClearAvi(void);

protected:
	void		Exception(CString stMsg) {
		AfxMessageBox(stMsg);
	}
};


#endif	//#define _INTEL_ICRC_AVSR_AVI_LOADER_H

