/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#ifndef	_INTEL_OPENCV_VISUAL_FEATURE_IO
#define _INTEL_OPENCV_VISUAL_FEATURE_IO

#include "cv.h"

#define AVSR_MFCC_AUDIO		70
#define AVSR_MFCC_VISUAL	99

typedef struct _AvsrMfccHeader {
	int		nSamples;		// number of samples in file (4-byte integer)
	int		samplePeriod;	// sample period in 100ns units (4-byte integer)
	short	sampleSize;		// number of bytes per sample (2-byte integer)
	short	paramKind;		// a code indicating the sample kind (2-byte integer)
} AvsrMfccHeader;


class CAvsrFeatureDataIO {

public:

	// for feature IO
	float *ReadFeatureFileMfcc(char szFilename[], AvsrMfccHeader &header);
	bool   WriteFeatureFileMfcc(char szFilename[], float pData[], AvsrMfccHeader header);
	bool   WriteFeatureFileMfcc(char szFilename[], double pData[], AvsrMfccHeader header);

	// for PCA
	bool LoadEigenMatrix(const char* szFilename, CvMat** avgMat, CvMat** eValue, CvMat** eVector, bool bBinary = true);
	bool SaveEigenMatrix(const char* szFileName, CvMat* avgMat, CvMat* eValue, CvMat* eVector, bool bBinary = true);

	// for LDA
	bool LoadLdaMatrix( const char* szFileName, CvMat** meanMat, CvMat** devsMat, 
						CvMat** evMat, CvMat** transMat, bool bBinary = true);
	bool SaveLdaMatrix( const char* szFileName, CvMat* meanMat, CvMat* devsMat, 
						CvMat* evMat, CvMat* transMat, bool bBinary = true);
	bool ReplaceFileExt(char szFileOld[], char szNewExt[], char szFileNew[]);

private:
	void	Exception(char szMsg[]);

};

#endif	// #ifndef	_INTEL_OPENCV_VISUAL_FEATURE_IO


