/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#include <stdio.h>
#include <direct.h>
#include <io.h>
#include <stdlib.h>  
#include <string.h> 
#include "cvVisualSpeech.h"
#include "cvAvsrFeatureIO.h"

FILE	*g_fpLdaLogFile = NULL;
bool	g_isLdaLogFileOk = false;

bool LdaTrain(char *visFile, char *labFolder, char *featFolder, char *matFile, int nLdaVecNum);
CvMat *UpsampleFmnConc(CvMat *pcaMat, float fUpRate, int nConcWndRadius);
bool LdaProj(char *ldaFile, char *featFolder, char *ldaoutFolder); 

void printusage(void)
{
	printf("LDA training:\tlda.exe -train <viseme file> <input data folder> <label folder> <LDA matrix file> <subspace dim>\n");
	printf("LDA projection:\tlda.exe -proj <LDA matrix file> <input data folder> <result file folder>\n");
	return;
}

int main(int argc, char *argv[])
{
	printf("\n--------- Viseme based LDA (AVSR group with ICRC) ------------\n");
	char visFile[256];
	char labFolder[256];
	char featFolder[256];
	char matFile[256];
	char ldaFile[256];
	char ldaoutFolder[256];
	
	// check argument
	if (argc != 7 && argc != 5) 
	{
		printf("Error: Invalid input argument!\n");
		printusage();
		return 0;
	}
	if(argc==7)
	{
		if(strcmp(strlwr(argv[1]),"-train")!=0)
		{
			printf("Error: Invalid input argument!\n");
			printusage();
			return 0;
		}
		// for increment training
		int nLdaVecNum;
		strncpy(visFile, argv[2], 255);
		strncpy(featFolder, argv[3], 255);
		strncpy(labFolder, argv[4], 255);
		strncpy(matFile, argv[5], 255);	
		sscanf(argv[6], "%d", &nLdaVecNum);
		// training
		LdaTrain(visFile, labFolder, featFolder, matFile, nLdaVecNum); 
	} else {	//argc==5
		if (strcmp(strlwr(argv[1]),"-proj")!=0)
		{
			printf("Error: Invalid input argument!\n");
			printusage();
			return 0;
		}
		// for increment training
		strncpy(ldaFile, argv[2], 255);
		strncpy(featFolder, argv[3], 255);
		strncpy(ldaoutFolder, argv[4], 255);
		// transform
		LdaProj(ldaFile, featFolder, ldaoutFolder); 
	}

	return 0;
}

// do lad training incremently
bool LdaTrain(char *visFile, char *labFolder, char *featFolder, char *matFile, int nLdaVecNum)
{
	const float fDstSampleRate = 100.0f;
	const int nConcWndRadius = 7;

	CvMat *Sw; // within-class scatter matrix
	CvMat *Sb; // between-class scatter matrix
	CvMat *feature_means; // total input feature means
	CvMat *sdevs; // total input feature standard deviation
	CvMat *transformMat; // transform matrix
	CvMat *eigenvalue;
	CvMat *class_trials;
	CvMat *input_sq_total;
	CvMat *input_total_per_class;		
	CvMat *class_cov[CV_LDA_MAX_CLASSES];

	int	pVisFreq[64];	
	int	vecSize;
	int nInVecSize;
	int	nClassNum;
	char pViseme[64][8]; 
	float fUpsampleRate;
	
	int type = CV_64F;
	long hFile = -1L;
	int classes = 0;	
	int i, j, nclass, beg, end;
	
	float *pInData;
	bool isFirstFile, isFirstProcessed;
	char sv[3][30];
	char szLabFolder[256];
	char szFeatFile[256];
	char szLabFile[256];
	char szTemp[256];
	FILE *pfVis, *lfn;
	struct _finddata_t c_file;
	CAvsrFeatureDataIO featureIo;

	// load viseme file
	for(i=0;i<64;i++) {
		for(j=0;j<8;j++) {
			pViseme[i][j]=0;
		}
	}
	pfVis = fopen(visFile,"rt");
	if (pfVis == NULL) {
		printf("Error: Can not open viseme file <%s>!\n", visFile);	
		return false;
	}
	while (!feof(pfVis)) {
		if (fscanf(pfVis, "%s", pViseme[classes]) != 1) continue;
		pVisFreq[classes] = 0;
		classes ++;
	}
	nClassNum = classes;

	// load feature data
	isFirstFile = true;
	isFirstProcessed = true;
	int featnum = 0;
	int totalfeature = 0;
	int processed = 0;	
	sprintf(szLabFolder, "%s\\*.vis", labFolder);
	if ((hFile = _findfirst(szLabFolder, &c_file )) == -1L ) 
	{
		printf("Error: Can't find viseme label file in the current folder.");
		return false;
	}
	do {
		// load feature data
		if (!featureIo.ReplaceFileExt(c_file.name, ".pmfc", szTemp)) {
			printf("Waring: Invalid file name <%s>.", c_file.name);
		}
		sprintf(szFeatFile, "%s\\%s", featFolder, szTemp);
		AvsrMfccHeader mfcc;
		pInData = featureIo.ReadFeatureFileMfcc(szFeatFile, mfcc);
		if (pInData == NULL) {
			//printf("Warning: Can not load data in file <%s>.\n", szTemp);
			continue;
		}
		// check vector size
		if (isFirstFile) {
			nInVecSize = mfcc.sampleSize / sizeof(float);
			isFirstFile = false;
		}
		else {
			if (mfcc.sampleSize / sizeof(float) != (unsigned int)nInVecSize) {
				printf("Warning: Invalid vector length in file <%s>! Skip...\n", szTemp);
				delete pInData;
				continue;
			}
		}
		// Upsample + FMN + concatenation
		printf("Process file <%s>... ", szFeatFile);
		CvMat pcaMat = cvMat(mfcc.nSamples, nInVecSize, CV_32F, pInData);
		fUpsampleRate = fDstSampleRate * mfcc.samplePeriod / 10000000;
		CvMat *featmat = UpsampleFmnConc(&pcaMat, fUpsampleRate, nConcWndRadius);
		delete pInData;
		if (featmat == NULL) continue;
		featnum = featmat->rows;
		vecSize = featmat->cols;
		// read viseme file and get matrix data
		sprintf(szLabFile, "%s\\%s", labFolder, c_file.name);
		lfn = fopen(szLabFile, "rt");
		if(lfn == NULL) {
			printf("Warning: Can not open label file <%s>!\n", c_file.name);
			cvReleaseMat(&featmat);
			continue;
		}
		// allocate label matrix
		CvMat *labmat = cvCreateMat(featnum, 1, CV_8U);
		if (labmat == NULL) {
			printf("Error: Can't allocate buffer for label data!\n");
			cvReleaseMat(&featmat);
			return false;
		}
		for (i=0; i<featnum; i++) {
			labmat->data.i[i] = -1;
		}
		while (!feof(lfn)) {
			fscanf(lfn,"%s %s %s\n",sv[0],sv[1],sv[2]);
			beg=atoi(sv[0]); 
			end=atoi(sv[1]);
			// get class of the data
			nclass = -1;
			for(int k=0; k<nClassNum; k++) {
				if(strcmp(sv[2], pViseme[k])==0) {
					nclass=k;
					break;
				}
			}			
			// check boundary of viseme
			if (nclass == -1 || beg < 0 || end > featnum || beg >= end) {
				printf("Warning: Unknown viseme or invalid boundary of viseme in <%s (%d, %d)>!\n", sv[2], beg, end);
				continue;
			}
			pVisFreq[nclass] ++;
			// set label
			for (i=beg; i<end; i++) {
				labmat->data.i[i] = nclass;
			}
		} //end of read viseme file
		fclose(lfn);
		// process
		if (isFirstProcessed) {
			// allocate matrices
			class_trials = cvCreateMat(1, nClassNum, type);
			input_sq_total = cvCreateMat(1, vecSize, type);
			input_total_per_class = cvCreateMat(nClassNum, vecSize, type);		
			for(int k = 0; k < nClassNum; k++ )
			{
				class_cov[k] = cvCreateMat(vecSize, vecSize, type);
			}
			// compute
			cvInitLDA(labmat, featmat, nClassNum, class_trials, input_total_per_class, class_cov, input_sq_total);
			isFirstProcessed = false;
		}
		else {			
			cvUpdateLDA(labmat, featmat, nClassNum, class_trials, input_total_per_class, class_cov, input_sq_total);
		}
		// release feature data
		cvReleaseMat(&featmat);
		cvReleaseMat(&labmat);
		processed ++;
		totalfeature += featnum;
	 	printf(" Ok!\n");
	} while (_findnext( hFile, &c_file ) == 0);

	if (processed < 1) {
		printf("Error: No .vis data file in the label folder <%s>!\n", labFolder);
		return false;
	}

	for (i=0; i<nClassNum; i++) { 
		if (pVisFreq[i] == 0) {
			printf("Error: One of viseme not used.\n");
			return false;
		}
	}

	// display result
	printf("%d data files used.\n", processed);
	for (i=0; i<nClassNum; i++) {
		printf("%02d\t%s\t%d\n", i, pViseme[i], pVisFreq[i]);
	}
	
	//compute the scatter matrices
	printf("Compute within-class and between scatter matrices...\n");
	Sw = cvCreateMat(vecSize, vecSize, type);	
	Sb = cvCreateMat(vecSize, vecSize, type);	
	feature_means = cvCreateMat(vecSize, 1, type);	
	sdevs = cvCreateMat(vecSize, 1, type);	
	cvCalcLDASwSb(class_trials, input_total_per_class, class_cov, input_sq_total, Sw, Sb, feature_means, sdevs);
	cvReleaseMat(&class_trials);
	cvReleaseMat(&input_sq_total);
	cvReleaseMat(&input_total_per_class);		
	for(int k = 0; k < nClassNum; k++ )	{
		cvReleaseMat(&class_cov[k]);
	}

	// compute the transform matrix
	printf("Compute the transform matrix...\n");
	transformMat = cvCreateMat(nLdaVecNum, vecSize, type);
	eigenvalue = cvCreateMat(nLdaVecNum, 1, type);
	cvCalcLDAMatrix(Sw, Sb, transformMat, eigenvalue);

	// normalize the transform matrix
	cvNormLDAMatrix( eigenvalue, transformMat );

	// save result
	printf("Save transform matrix...\n");
	featureIo.SaveLdaMatrix(matFile, feature_means, sdevs, eigenvalue, transformMat, false); // save text file
	printf("Training complete!\n");	

	cvReleaseMat(&Sw);
	cvReleaseMat(&Sb);	
	cvReleaseMat(&feature_means);
	cvReleaseMat(&sdevs);
	cvReleaseMat(&transformMat);

	return true;
}

bool LdaProj(char *ldaFile, char *featFolder, char *ldaoutFolder)
{
	const int type = CV_32F;
	const float fAmpFactor = 5000.0f;
	const float fDstSampleRate = 100.0f;
	const int nConcWndRadius = 7;
	// load LDA matrix
	CAvsrFeatureDataIO featureIo;
	CvMat* means = NULL;	// feature means
	CvMat* sdevs = NULL;	// feature MSE
	CvMat* evMat = NULL;	// eigen values corresponding to transform matrix
	CvMat* tranMat = NULL;	// transform matrix
	if(!featureIo.LoadLdaMatrix(ldaFile, &means, &sdevs, &evMat, &tranMat, false)){	// load text lda file
	// for debug here
	//if(!featureIo.LoadLdaMatrix1(ldaFile, &means, &sdevs, &evMat, &tranMat, false)){	// load text lda file
		printf("Error: Fail to load LDA matrix data!\n");
		return false;
	}
	int nLdaVecSizeOut = tranMat->rows;
	int nLdaVecSizeIn = tranMat->cols;

	float *pInData, fUpsampleRate;
	char szFeatFile[256];
	int nVecSize, nPcaVecSize;

	// create output folder
	_mkdir(ldaoutFolder);

	// process data
	long hFile = -1L;
	struct _finddata_t c_file;
	int  nProcessed = 0;
	char szFolder[256];
	sprintf(szFolder, "%s\\*.pmfc", featFolder);
	bool isFirstFile = TRUE;
	if ((hFile = _findfirst(szFolder, &c_file )) == -1L ) {
		printf("Can't find data file in the given folder <%s>!\n", szFolder);
		cvReleaseMat(&means);
		cvReleaseMat(&sdevs);
		cvReleaseMat(&evMat);
		cvReleaseMat(&tranMat);
		return false;
	}
	do {
		// load input data
		sprintf(szFeatFile, "%s\\%s", featFolder, c_file.name);
		printf("Processing file <%s>...", szFeatFile);
		AvsrMfccHeader mfcc;
		pInData = featureIo.ReadFeatureFileMfcc(szFeatFile, mfcc);
		if (pInData == NULL) {
			//printf("Warning: Can not load data in file <%s>.\n", szFeatFile);
			continue;
		}
		// check vector size
		if (isFirstFile) {
			nPcaVecSize = mfcc.sampleSize / sizeof(float);
			isFirstFile = false;
		}
		else {
			nVecSize = (mfcc.sampleSize * (nConcWndRadius * 2 + 1)) / sizeof(float);
			if (nVecSize != nLdaVecSizeIn) {
				printf("Warning: Invalid vector length in file <%s>! Skip...\n", c_file.name);
				delete pInData;
				continue;
			}
		}
		// Upsample + FMN + concatenation
		CvMat pcaMat = cvMat(mfcc.nSamples, nPcaVecSize, CV_32F, pInData);
		fUpsampleRate = fDstSampleRate * mfcc.samplePeriod / 10000000;
		CvMat *featMat = UpsampleFmnConc(&pcaMat, fUpsampleRate, nConcWndRadius);
		delete pInData;
		if (featMat == NULL) continue;
		CvMat *ldaoutMat = cvCreateMat(featMat->rows, nLdaVecSizeOut, type);		
		//if (cvLdaTransform(tranMat, means, sdevs, featMat, fAmpFactor, ldaoutMat) != CV_StsOk) 
		if (cvLDAProjection(tranMat, featMat, ldaoutMat) != CV_StsOk) 
		{
			printf("Error: LDA transformation failed!\n");
			continue;
		}
		int i;
		if (cvGetElemType(ldaoutMat) == CV_32F) {
			for (i=0; i<ldaoutMat->cols*ldaoutMat->rows; i++) {
				ldaoutMat->data.fl[i] *= fAmpFactor;
			}
		}
		else {
			for (i=0; i<ldaoutMat->cols*ldaoutMat->rows; i++) {
				ldaoutMat->data.db[i] *= fAmpFactor;
			}
		}

		// convert output data to float
		float* pRlt = new float[ldaoutMat->rows*ldaoutMat->cols];
		if (pRlt==NULL) {
			printf("Error: Can not allocate lda result matrix! Not enough memory!");
			cvReleaseMat(&featMat);
			cvReleaseMat(&ldaoutMat);
			continue;
		} else {
			if(type == CV_32F) {
				memcpy(pRlt, ldaoutMat->data.fl, ldaoutMat->rows*ldaoutMat->cols * sizeof(float));	
			} else {
				for (int i=0; i<ldaoutMat->rows*ldaoutMat->cols; i++) {
					pRlt[i] = (float)(ldaoutMat->data.db[i]);
				}
			}
		}
		cvReleaseMat(&featMat);
		// save result
		AvsrMfccHeader header;
		header.nSamples = ldaoutMat->rows;
		header.sampleSize = ldaoutMat->cols * sizeof(float);
		header.samplePeriod = (int)(10000000 / fDstSampleRate);;
		header.paramKind = AVSR_MFCC_VISUAL;
		char szMfccFile[256];
		featureIo.ReplaceFileExt(c_file.name, ".mfcc", szMfccFile);
		char szRltFile[256];
		sprintf(szRltFile, "%s\\%s", ldaoutFolder, szMfccFile);
		featureIo.WriteFeatureFileMfcc(szRltFile, pRlt, header);
		//featureIo.WriteFeatureFileText(szRltFile, pRlt, ldaoutMat->cols, header.nSamples, header.samplePeriod);
		nProcessed ++;
		cvReleaseMat(&ldaoutMat);
		delete pRlt;
		printf("Ok\n");
	} while (_findnext( hFile, &c_file ) == 0);

	printf("%d files processed!", nProcessed);
	
	cvReleaseMat(&means);
	cvReleaseMat(&sdevs);
	cvReleaseMat(&evMat);
	cvReleaseMat(&tranMat);
	
	return true;
}

CvMat *UpsampleFmnConc(CvMat *pcaMat, float fUpRate, int nConcWndRadius)
{
	if (pcaMat == NULL || fUpRate < 1.0f || nConcWndRadius < 1) {
		printf("Error: Invalid input argument!\n");
		return NULL;
	}

	int nInVecSize = pcaMat->cols;
	int nInSmpNum = pcaMat->rows;
	// upsample
	int nOutSmpNum = (int)(nInSmpNum * fUpRate + 0.5f);
	CvMat *upsample = cvCreateMat(nOutSmpNum, nInVecSize, CV_32F);
	if (upsample == NULL) {
		printf("Error: Fail to allocate matrix for upsampled feature data!\n");
		return NULL;
	}
	if (cvInterpolateFT(pcaMat, fUpRate, upsample) != CV_StsOk) {
		printf("Error: Fail to upsample the data!\n");
		cvReleaseMat(&upsample);
		return NULL;
	}
	// FMN
	if( cvNormalizeMean(upsample)!= CV_StsOk) {
		printf("Error: Fail to do FMN!\n");
		cvReleaseMat(&upsample);
		return NULL;
	}
	// concatenation
	int nWndWidth = 2 * nConcWndRadius + 1;
	int nOutVecSize = nInVecSize * nWndWidth;
	CvMat *conc = cvCreateMat(nOutSmpNum, nOutVecSize, CV_32F);
	if (conc == NULL) {
		printf("Error: Fail to allocate matrix for concatenated feature data!\n");
		cvReleaseMat(&upsample);
		return NULL;
	}	
	if( cvConcatVectors(upsample, nWndWidth, conc) != CV_StsOk){
		printf("Error: Fail to do concatenation!\n");
		cvReleaseMat(&upsample);
		cvReleaseMat(&conc);
		return NULL;
	}
	
	cvReleaseMat(&upsample);

	return conc;
}
