/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#ifndef	ICV_VISUAL_SPEECH_H
#define	ICV_VISUAL_SPEECH_H

#include "_cv.h"
#include "_cvaux.h"
#include "cvtypes.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef CV_LDA_SMALLEST 
#define CV_LDA_SMALLEST 1.e-30
#endif

#ifndef CV_LDA_MAX_RATIO 
#define CV_LDA_MAX_RATIO 1.0e5
#endif

#ifndef CV_LDA_MAX_CLASSES
#define CV_LDA_MAX_CLASSES 64
#endif

OPENCVAUXAPI CVStatus  
cvConcatVectors( const CvMat* inVectors, const int J, CvMat* outVectors );

OPENCVAUXAPI  CVStatus  
cvNormalizeMean( CvMat* inVectors );

// visual feature upsampling
OPENCVAUXAPI void  
icvFT1D( const CvMat* inData, CvMat* outRe, CvMat* outIm );

OPENCVAUXAPI void  
icvIFT1D( const CvMat* inRe, const CvMat* inIm, CvMat* outData );

OPENCVAUXAPI CVStatus  
cvInterpolateFT( const CvMat* inVectors, const float R, CvMat* outVectors );

// PCA functions
OPENCVAUXAPI  CVStatus 
cvInitPCA(const IplImage** srcImg, const int blockImgNum, CvMat* sumVector, CvMat* pCovMatrix);

OPENCVAUXAPI  CVStatus 
cvUpdatePCA(const IplImage** srcImg, const int blockImgNum, int &totalImgNum, 
			CvMat* sumVector, CvMat* pcovMatrix);

OPENCVAUXAPI  CVStatus
cvCalcPCAMatrix( const int totalImgNum, const CvMat* sumVector, const CvMat* pcovMatrix,
				 CvMat* meanVector, CvMat* covMatrix, CvMat* eigenValues, CvMat* eigenVectors);

OPENCVAUXAPI  CVStatus
cvPCAProjection(const IplImage** srcImg, const int imgNum, 
				const CvMat* eigenValues, const CvMat* eigenVectors, const CvMat* meanVector, 
				CvMat* projVectors, bool eigenNorm  CV_DEFAULT(false));

// LDA functions
OPENCVAUXAPI  CVStatus 
cvInitLDA( const CvMat* labels, const CvMat* inVectors, const int numClasses, 
           CvMat *numSamplesInClass, CvMat *sumVectorInClass, 
           CvMat **pCovMatrixInClass, CvMat *normFactor);


OPENCVAUXAPI  CVStatus 
cvUpdateLDA( const CvMat* labels, const CvMat* inVectors, const int numClasses, 
			 CvMat *numSamplesInClass, CvMat *sumVectorInClass, 
			 CvMat **pCovMatrixInClass, CvMat *normFactor);


OPENCVAUXAPI  CVStatus 
cvCalcLDASwSb( const CvMat* numSamplesInClass, const CvMat* sumVectorInClass, 
			   CvMat** pCovMatrixInClass, const CvMat* normFactor,
               CvMat* withinClassScatter, CvMat* betweenClassScatter,
               CvMat* meanVectors, CvMat* mse);


OPENCVAUXAPI  CVStatus 
cvCalcLDAMatrix( CvMat* withinClassScatter, CvMat* betweenClassScatter, 
				 CvMat* projMat, CvMat* eigenvalues);


OPENCVAUXAPI  CVStatus 
cvNormLDAMatrix( const CvMat* eigenvalues, CvMat* projMat ); 

OPENCVAUXAPI  CVStatus 
cvLDAProjection( const CvMat* projMatrix, const CvMat* inVectors, CvMat* outVectors );


#ifdef __cplusplus
}
#endif

#endif


